/*
pitacard: A flashcard program
Copyright (C) 2003 Austin Bingham

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach the author at:
    abingham@spamcop.net
*/

#include "ktreader.h"

#include "keyTranslator.h"
#include "util.h"

#include <iostream>

#include <qfile.h>
#include <qnamespace.h>
#include <qstringlist.h>
#include <qtextstream.h>

using namespace std;

namespace
{

    using namespace pitacard::xlat;
    
    enum Mode
    {
	HEADER,
	TRANSLATIONS,
	UNKNOWN
    };
    
    int translateModifiers(const QString& input)
    {
	QStringList sl =
	    QStringList::split(" ",
			       input.stripWhiteSpace(),
			       false);

	int rval = Qt::NoButton;

	for (QStringList::size_type idx = 0; idx < sl.size(); ++idx)
	{
	    rval |= pitacard::util::translateQtButton(sl[idx]);
	}

	return rval;
    }

    QString translateUnicode(const QString& input)
    {
	QStringList sl =
	    QStringList::split(" ",
			       input.stripWhiteSpace(),
			       false);

	QString rval;

	for (QStringList::size_type idx = 0; idx < sl.size(); ++idx)
	{
	    rval += pitacard::util::translateUnicodeChar(sl[idx]);
	}

	return rval;
    }

    bool newSection(const QString& line, Mode& mode)
    {
	QString sz = line.stripWhiteSpace();
	
	if (sz == "[header]")
	{
	    mode = HEADER;
	    return true;
	}
	else if (sz == "[translations]")
	{
	    mode = TRANSLATIONS;
	    return true;
	}
	
	return false;
    }

    void handleHeader(const QString& line, Header& hdr)
    {
	if (line.length() < 1)
	{
	    cerr << "[handleHeader] ERROR: Header line must contain at least one character" << endl;
	    return;
	}
	
	QStringList sl = QStringList::split(' ', line, true);
	QString name = sl[0];
	sl.pop_front();
	hdr[name] = sl.join(" ");
    }
    
    void handleTranslation(const QString& line, KeyTranslator& kt)
    {
	QStringList slist = QStringList::split(",", line, true);
	
	if (slist.size() != 3)
	{
	    cerr << "[handleTranslation] ERROR: Unable to parse line: "
		 << line << endl;
	    return;
	}
	
	try
	{
	    Qt::Key key = pitacard::util::translateQtKey(slist[0]);
	    int mod = translateModifiers(slist[1]);
	    QString unicode = translateUnicode(slist[2]);
	    
	    kt.insert(key, mod, unicode);
	}
	catch (QString& sz)
	{
	    cerr << "[handleTranslation] ERROR: Unable to parse line: "
		 << line << endl
		 << "\tREASON: " << sz << endl;
	}
    }
    
}

namespace pitacard { namespace xlat {

    void readKeyTranslator(QIODevice& iod,
			   KeyTranslator& kt,
			   Header& hdr)
    {
	Mode mode = UNKNOWN;
	
	QTextStream input(&iod);
	
	while (!input.atEnd())
	{
	    QString line = input.readLine();
	    
	    // Strip comment
	    int commentIdx = line.find('#');
	    if (commentIdx != -1)
		line = line.left(commentIdx);

	    line = line.simplifyWhiteSpace();

	    if (line.length() < 1) continue;

	    Mode newMode;
	    if (newSection(line, newMode))
		mode = newMode;
	    else
	    {
		switch (mode)
		{
		    case HEADER:
			handleHeader(line, hdr);
			break;
			
		    case TRANSLATIONS:
			handleTranslation(line, kt);
			break;

		    default:
			break;
		}
	    }
	}
    }

    void readKeyTranslator(const QString& filename,
			   KeyTranslator& kt,
			   Header& hdr)
    {
	QFile file(filename);
	if (!file.open(IO_ReadOnly))
	{
	    std::cerr << "[readKeyTranslator] ERROR: Unable to open file: "
		 << filename << std::endl;
	    return;
	}
	
	readKeyTranslator(file, kt, hdr);
    }
    
} }
