#include "DeckEditorUI.h"

#include "AboutDlg.h"
#include "DeckListView.h"
#include "deckreader.h"
#include "deckwriter.h"
#include "globals.h"
#include "KeyboardDeckRunnerUI.h"
#include "randomSelector.h"
#include "util.h"

#include <iostream>

#include <qaction.h>
#include <qapplication.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qfontdialog.h>
#include <qmessagebox.h>

using namespace pitacard::data;
using namespace pitacard::util;
using namespace pitacard::xlat;

namespace {

    /** This is hackery...QListView and QListViewItem support the same interface
	in terms of iterations/crawling, so I'm exploiting that to avoid
	code duplication */
    template <class In>
    void getFolder(In* in, Folder& f)
    {
	using namespace pitacard::ui;
	
	for (QListViewItem* item = in->firstChild();
	     item != NULL;
	     item = item->nextSibling())
	{
	    if (CardListViewItem* clvi =
		dynamic_cast<CardListViewItem*>(item))
	    {
		f.mCards.push_back(clvi->card());
	    }
	    else if (FolderListViewItem* flvi =
		     dynamic_cast<FolderListViewItem*>(item))
	    {
		f.mFolders.push_back(Folder());
		f.mFolders.back().mName = flvi->name();
		getFolder(flvi, f.mFolders.back());
	    }
	}
    }

    template <class In>
    void addFolder(const Folder& f, In where)
    {
	using namespace pitacard::ui;
	
	for (Folder::Cards::const_iterator itr = f.mCards.begin();
	     itr != f.mCards.end();
	     ++itr)
	{
	    new CardListViewItem(Card(itr->name(),
				      itr->front().mText,
				      itr->back().mText),
				 where);
	}

	for (Folder::Folders::const_iterator itr = f.mFolders.begin();
	     itr != f.mFolders.end();
	     ++itr)
	{
	    FolderListViewItem* newFolder =
		new FolderListViewItem(itr->mName, where);
	    addFolder(*itr, newFolder);
	}
    }
    
}

namespace pitacard { namespace ui {
    
    DeckEditorUI::DeckEditorUI(QWidget* parent,
			       const char* name,
			       WFlags fl) :
	DeckEditorUIBase (parent, name, fl),
	mUnsavedChanges (false)
    {
	using namespace pitacard::config;

	// TODO: This needs to not be hardcoded to greek
	//mFrontTextEdit->setKeyTranslator(pitacard::config::translators()["greek"]);

	{
	    using namespace pitacard::config;
	    
	    QStringList sl;
	    for (XlatMap::const_iterator itr = translators().begin();
		 itr != translators().end();
		 ++itr)
	    {
		sl.push_back(itr->first);
	    }

	    mFrontXlatorCombo->insertStringList(sl);
	    mBackXlatorCombo->insertStringList(sl);

	    mFrontXlatorCombo->insertItem("NONE", 0);
	    mBackXlatorCombo->insertItem("NONE", 0);
	}
	
	mFrontTextEdit->setFont(cardFont());
	mBackTextEdit->setFont(cardFont());
    }

    void DeckEditorUI::fileNew()
    {
	if (!queryUnsavedChanges()) return;
	
	mCardList->clear();
	mFilename = "";

	mUnsavedChanges = false;
	fileSaveAction->setEnabled(false);

	mFrontTextEdit->setText("");
	mBackTextEdit->setText("");

	mFrontTextEdit->setEnabled(false);
	mBackTextEdit->setEnabled(false);
    }
    
    void DeckEditorUI::fileOpen()
    {
	using namespace pitacard::data;

	if (!queryUnsavedChanges()) return;
	
	QString s = QFileDialog::getOpenFileName();

	if (s.length() < 1) return;

	mFilename = s;

	mCardList->clear();
	
	mUnsavedChanges = false;
	fileSaveAction->setEnabled(false);
	
	mFrontTextEdit->setText("");
	mBackTextEdit->setText("");
	
	mFrontTextEdit->setEnabled(false);
	mBackTextEdit->setEnabled(false);

	Folder folder;
	pitacard::io::readDeck(s, folder);

	addFolder(folder, mCardList);
    }

    void DeckEditorUI::fileSave()
    {
	if (mFilename.length() < 1)
	    fileSaveAs();
	else
	{
	    Folder f;
	    getFolder(mCardList, f);
	    pitacard::io::writeDeck(mFilename, f);
	    mUnsavedChanges = false;
	    fileSaveAction->setEnabled(false);
	}
    }
    
    void DeckEditorUI::fileSaveAs()
    {
	QString s =
	    QFileDialog::getSaveFileName(
		".",
		"All Files (*)",
		this,
		"save file dialog"
		"Choose a filename to save as");

	if (s.length() < 1) return;

	mFilename = s;
	fileSave();
    }

    void DeckEditorUI::fileExit()
    {
	if (!queryUnsavedChanges()) return;
	
	qApp->exit();
    }
    
    void DeckEditorUI::deckAddCard()
    {
	mCardList->addCard();
    }
    
    void DeckEditorUI::deckRemoveCard()
    {
	mCardList->removeCard();
    }

    void DeckEditorUI::deckAddFolder()
    {
	mCardList->addFolder();
    }

    void DeckEditorUI::deckRemoveFolder()
    {
	mCardList->removeFolder();
    }
    
    void DeckEditorUI::deckReview()
    {
	Folder f;
	getFolder(mCardList, f);
	KeyboardDeckRunnerUI<RandomSelector> drui(f,
						  this,
						  "DeckRunnerUI",
						  true);
	drui.exec();
    }

    void DeckEditorUI::helpAbout()
    {
	AboutDlg aboutDlg(this, "about-dlg", true);
	aboutDlg.exec();
    }
    
    void DeckEditorUI::setCardFont()
    {
	bool ok;
	QFont font =
	    QFontDialog::getFont(&ok,
				 pitacard::config::cardFont(),
				 this);
	if (ok)
	{
	    pitacard::config::setCardFont(font);
	    mFrontTextEdit->setFont(pitacard::config::cardFont());
	    mBackTextEdit->setFont(pitacard::config::cardFont());
	}
    }
    
    void DeckEditorUI::deckSelectionChanged(QListViewItem* lbi)
    {
	QListViewItem* sel = mCardList->selectedItem();
	
	if (!sel)
	{
	    mFrontTextEdit->setText("");
	    mBackTextEdit->setText("");
	    
	    mFrontTextEdit->setEnabled(false);
	    mBackTextEdit->setEnabled(false);
	}
	else if (FolderListViewItem* flvi = dynamic_cast<FolderListViewItem*>(sel))
	{
	    mFrontTextEdit->setText("");
	    mBackTextEdit->setText("");
	    
	    mFrontTextEdit->setEnabled(false);
	    mBackTextEdit->setEnabled(false);
	}
	else if (CardListViewItem* clvi = dynamic_cast<CardListViewItem*>(sel))
	{
	    mFrontTextEdit->setText(clvi->card().front().mText);
	    mBackTextEdit->setText(clvi->card().back().mText);
	    
	    mFrontTextEdit->setEnabled(true);
	    mBackTextEdit->setEnabled(true);
	}
    }

    void DeckEditorUI::frontTextChanged()
    {
	CardListViewItem* clvi = mCardList->currCard();
	
	if (!clvi) return;

	if (clvi->card().front().mText != mFrontTextEdit->text())
	{
	    Card c = clvi->card();
	    c.front().mText = mFrontTextEdit->text();
	    clvi->setCard(c);

	    mUnsavedChanges = true;
	    fileSaveAction->setEnabled(true);
	}
    }

    void DeckEditorUI::backTextChanged()
    {
	CardListViewItem* clvi = mCardList->currCard();
	
	if (!clvi) return;

	if (clvi->card().back().mText != mBackTextEdit->text())
	{
	    Card card = clvi->card();
	    card.back().mText = mBackTextEdit->text();
	    clvi->setCard(card);

	    mUnsavedChanges = true;
	    fileSaveAction->setEnabled(true);
	}
    }

    void DeckEditorUI::frontXlatorChanged(int idx)
    {
	using namespace pitacard::config;

	if (idx < 1)
	{
	    mFrontTextEdit->setKeyTranslator(KeyTranslator());
	}
	else
	{
	    assert(translators().find(mFrontXlatorCombo->text(idx)) != translators().end());
	    
	    mFrontTextEdit->setKeyTranslator(translators()[mFrontXlatorCombo->text(idx)]);
	}
    }
    
    void DeckEditorUI::backXlatorChanged(int idx)
    {
	using namespace pitacard::config;

	if (idx < 1)
	{
	    mBackTextEdit->setKeyTranslator(KeyTranslator());
	}
	else
	{
	    assert(translators().find(mBackXlatorCombo->text(idx)) != translators().end());
	    
	    mBackTextEdit->setKeyTranslator(translators()[mBackXlatorCombo->text(idx)]);
	}
    }
    
    void DeckEditorUI::deckChanged()
    {
	mUnsavedChanges = true;
	fileSaveAction->setEnabled(true);

	CardListViewItem* clvi = mCardList->currCard();
	
	mFrontTextEdit->setText(clvi ?
				clvi->card().front().mText :
				QString(""));
	
	mBackTextEdit->setText(clvi ?
			       clvi->card().back().mText :
			       QString(""));
	
	mFrontTextEdit->setEnabled(clvi);
	mBackTextEdit->setEnabled(clvi);
    }

    bool DeckEditorUI::queryUnsavedChanges()
    {
	if (!mUnsavedChanges) return true;

	switch (QMessageBox::warning(this,
				     "Unsaved changes",
				     "Do you wish to save your changes?",
				     QMessageBox::Yes,
				     QMessageBox::No,
				     QMessageBox::Cancel))
	{
	    case QMessageBox::Yes:
		fileSave();
		return true;
	    case QMessageBox::No:
		return true;
	    case QMessageBox::Cancel:
		return false;
	}
    }

} }
